#*******************************************************************************
# E.S.O. - VLT project
#
# "@(#) $Id: SkyCat_plugin.tcl,v 2.43 2004/06/17 23:04:30 vltsccm Exp $"
#
# who       when      what
# --------  --------  ----------------------------------------------
# hummel  13/08/99  created
# aaguayo 14/03/01  fims 2.28 (use of SKYCAT_VER variable) 
# mkiekebu 31/03/01 removed bug (last modification)
# mkiekebu 05/19/01 added rtd plugin to overload File-Open
# mkiekebu 06/11/01 added rtd plugin to overload File-Exit
# aaguayo  28/10/03 PPRS 10127 - fsmosaic considered for both FORSes
#
#
#************************************************************************
#   NAME
#      SkyCat_plugin - SkyCat startup file for plug-ins
#
#   SYNOPSIS
#      SkyCat_plugin {this}
#
#   DESCRIPTION
#      This is an example plugin file for skycat. It can be used to add
#      features to skycat at startup. To use this file for skycat, copy it
#      to $HOME/.skycatrc or set the environment variable SKYCATRC to the
#      path name of the file.
#
#      This file must define at least one tcl proc: SkyCat_plugin, which
#      takes one argument: the name of the top level skycat widget. 
#      You can refer to the man pages and source code for the Skycat and
#      RTD Itcl widgets for details.
#
#      FORS particularities:
#      several SkyCat, methods are extended:
#      RtdCtrl
#      RtdImageTrans
#
#   FILES
#
#   ENVIRONMENT
#      HP-UX-10.20, ESO SkyCat Tool
#
#   COMMANDS
#       SkyCat_plugin {this} 
#       add_help_feature {w} 
#              adds 4 more buttons in the help menu
#       add_test_menubutton {w} 
#              maintenance 
#       test_callback {w} 
#              maintenance 
#       add_fors1_menu {w} 
#              adds the 'FORS1' cascaded main menu and all submenus 
#              to the SkyCat Tool menu bar  
#       add_fors2_menu {w} 
#              adds the 'FORS2' cascaded main menu and all submenus 
#              to the SkyCat Tool menu bar  
#       f_on  {w f_number mode m} 
#              calls the fors::start_FORS class proc to create one single 
#              instance of FORS
#              configures all necessary menu buttons 
#       f_off {w f_number mode m} 
#              deletes the only instance of FORS
#              configures all necessary menu buttons 
#       f_con_sav {m} 
#              gets the number of exposures and puts it to the
#              VLT-FORS / Save button
#              no longer used 
#       grism_about {w_} 
#              calls the grism info widget
#       fors_about {w_} 
#              calls the fors info widget
#       
#   
#   RETURN VALUES
#
#   CAUTIONS
#       HITS mode in connection with highres collimator 
#       is allowed but not senseful
#
#   EXAMPLES
#
#   SEE ALSO
#
#   BUGS
#
#------------------------------------------------------------------------
#


# do not forget to 
# 'auto_mkindex /home/hummel/fims_plugin *.tcl .*rc' 
#
# This proc is required. It will be called once for each skycat
# instance. The parameter is the name ("$this") of the SkyCat
# class object.

puts -nonewline stdout "Loading the FIMS plugin..."

proc getErrorStr {errcode} {
    switch $errcode {
	2 {return "This is not a file"}
	3 {return "File has no permissions"}
	4 {return "This is not a fits file"}
	5 {return "Saving error"}
	6 {return "This file is a mosaic"}
	7 {return "Same file"}
	8 {return "Fits Error"}
	9 {return "Missing keyword"}
	10 {return "Slave not found"}
	11 {return "Master not found"}
	12 {return "Unknown CCD ID"}
	13 {return "Wrong keyword value"}
	14 {return "Align error"}
	15 {return "No instrument keyword"}
	16 {return "Unknown instrument"}
	default {return "Unknown error"}
    }
}

body rtd::Rtd::quit {} {
    global imageName

    puts "quitting fims application ..."
    if {[info exists imageName]} {
	catch {exec rm $imageName}
    }
    delete object $this
    after idle exit
}

body rtd::RtdImageCtrl::open {{dir "."} {pattern "*.*fit*"}} {
    global env imageName
    set w [utilNamespaceTail $this]
    set prep_dir $env(PREP_DIR)
    set file [filename_dialog $prep_dir $pattern $w]
    set which_fors $env(FORS_INST)

    if {"$file" != ""} {
	if {[file isfile $file]} {
	    detach_camera
		# AAG - PPRS 10127
		# fsmosaic considered for both FORSes
		set sec [clock seconds]
		set name [format "fsmosaic_%d.fits" $sec]
	        set errcode [system fsmosaic $file $prep_dir/$name]                                
                if {$errcode == 0} {
		    if {[info exists imageName]} {
			catch {exec rm $imageName}
		    }
		    puts "Loading merged image ..."		    
	            config -file "$prep_dir/$name"
		    set imageName $prep_dir/$name
		} elseif {($errcode == 6)} {	           
		    puts "This is not a mosaic image ..."
		    config -file $file 		   
		} elseif {($errcode == 12) || ($errcode == 15) || ($errcode == 16)} {
		    puts "This is not a mosaic image ..."
		    config -file $file 		   
		} else {
		    error [getErrorStr $errcode]
		}
		#
	} else {
	    error_dialog "There is no file named '$file'" $w_
	}
    }
    
}

# --------------------------
proc SkyCat_plugin {this} {
# --------------------------
    global skycat_plugin_initialized_ ::auto_path ::env

    # get the toplevel widget name
    set w [utilNamespaceTail $this]

    # add a new "Test" menu button
    # demo 
    # add_test_menubutton $w


    set which_fors $env(FORS_INST)
      switch -exact -- $which_fors  {
          2 { add_fors2_menu $w 
            puts stdout "for FORS2"
            } 
          1 { add_fors1_menu $w 
            puts stdout "for FORS1"
            }
          0 {  
            add_fors1_menu $w 
            add_fors2_menu $w 
            puts stdout "for FORS1+2"
            }
    default {puts stdout "Warning: $which_fors" }
           }

    # add a new FORS-HELP menu button 
    add_help_feature $w
    
    # this code should run only once
    # puts stdout ![info exists skycat_plugin_initialized_]
    if {! [info exists skycat_plugin_initialized_]} {
       # put here the body commands
       
       # disable the skycat pickobject button 
       set p [$w component view]
       $p.m  entryconfigure 3 -state disabled

       #--------------
       # RtdImageTrans
       #--------------
       # override flip/Z/z buttons
       set t [$w component image component info component trans]

       
       # ZOOM BUTTONS [Z] [z]
       # --------------------
       set body [$t info body update_trans]
       body rtd::RtdImageTrans::update_trans {} "$body\n \
       catch {fors::FORS::my_mask update_and_replot\n} "

#      catch {fors::FORS::my_mask update_scale}\n 

       # ROTATE BUTTON [@]
       # -----------------
       set body [$t info body rotate]
       body rtd::RtdImageTrans::rotate {} "$body\n \
       catch {fors::FORS::my_mask init_mask_world2}\n  \
       catch {fors::FORS::my_mask update_and_replot} "
 
       # FLIP BUTTON [><] [^v]
       # ---------------------
       set body [$t info body flip]
       body rtd::RtdImageTrans::flip {xy} "$body\n \
       catch {fors::FORS::my_mask init_mask_world2}\n  \
       catch {fors::FORS::my_mask update_and_replot} "
       
       # disable some skycat buttons no longer required
       # $t.rotate configure -state disabled 
       # $t.flipx  configure -state disabled 
       # $t.flipy  configure -state disabled 

       #--------
       # RtdCtrl
       #--------
       # override open/file 
       #set t [$w component image]
       #set body [$t info body open]
       #set prep_dir $env(PREP_DIR)
      
	#set file {}
	#set pattern "*.*fit*"
        
      # body rtd::RtdImageCtrl::open {{dir "."} {pattern "*.*fit*"}} \
      # "global .fs\n\
      #  catch { destroy .fs }\n\
      #  set file [filename_dialog $prep_dir $pattern $w]\n\
#	puts \"FILE: $file\"\n\
#	catch {fors::FORS::my_mask reset_mask\n\
 #                      $t.rotate configure -state disabled\n\
 #                       $t.flipy  configure -state disabled}\n"
  #      }

       set  skycat_plugin_initialized_ 1

    }

#-----------------------------
proc add_help_feature {w} {
#-----------------------------
     # This proc adds two menus to the Help menu
     # get the path name of the Help menu
     # see ToplevelWidget tclutil
     #set m [$w get_menu Help]
     set m [$w component menubar]
     set m $m.help.m
     #puts stdout $m

    $m add separator

    $w add_menuitem $m command "About FIMS..." \
       {Display a window with information about this FIMS version} \
       -command [code fors_about $w]

    $w add_menuitem $m command "FORS User Manual" \
       {Display information about FoRS in netscape (if netscape is available)} \
       -command [code $w.image  send_to_netscape \
       "http://www.eso.org/instruments/fors1/userman/userman.html"]

    $w add_menuitem $m command "FORS Exposure Time Calc" \
       {WWW connection to ESO ETC (if netscape is available)} \
       -command [code $w.image  send_to_netscape \
       "http://www.eso.org/observing/etc/"]

    $w add_menuitem $m command "Grism Info" \
       {Display information about FORS1 grism- OS filter combinations} \
       -command [code grism_about $w] 

    $w add_menuitem $m command "Mask Info" \
       {Display information about Standard Masks} \
       -command [code masks_about $w]  -state normal 

}


# add a menu button to the skycat menubar with some example menu items.
# $w is the name of the TopLevelWidget containing the menubar. 

#-----------------------------
proc add_test_menubutton {w} {
#-----------------------------
    # Allen's example 
    set m [$w add_menubutton "Test"]

    $w add_menuitem $m command "Test command" \
	{Test adding a command menu item} \
	-command [code test_callback $w]
    
    $m add separator

    $w add_menuitem $m checkbutton "Test checkbutton" \
	{Test adding a checkbutton menu item} \
	-variable test -onvalue 1 -offvalue 0 \
	-command [code test_callback $w]

    $w add_menuitem $m checkbutton "OPTIC" \
	{Test adding a checkbutton menu item} \
	-variable test -onvalue 1 -offvalue 0 \
	-command [code fors::FORS::my_mask test_optic_distor_FORS ]

    $w add_menuitem $m checkbutton "mask dump " \
	{Test adding a checkbutton menu item} \
	-variable test -onvalue 1 -offvalue 0 \
	-command [code fors::FORS::my_mask save_mask ]

    $w add_short_help $w.menubar.test \
	"Test menu: added in .skycatrc"
}


# dummy callback for menu items
#-----------------------
proc test_callback {w} {
#-----------------------
    puts "called test_callback $w"
    # we will use that dummy for lots of info commands
    # haha
    # get all components of $w = .skycat
    set sub [$w component]
    puts stdout "$w component = $sub" 
    puts stdout "[ [$w component image] cget -file]"
    
    foreach k $sub {
            puts stdout "$w component $k = [$w component $k]"

            set a [$w component $k]
            catch { set subsub [$a component]
            foreach g $subsub {
            puts stdout "$a component $g = [$a component $g]"
            }
            } 
           }

    

    set soso [$w component image]
    puts stdout "$soso get_image  = [$soso get_image]"
    puts stdout "$soso get_canvas = [$soso get_canvas]"
    puts stdout "$soso get_imageId = [$soso get_imageId]"
    # puts stdout "$soso show_toolbox = [$soso show_toolbox]"
    # puts stdout "$soso spectrum = [$soso spectrum]"
    # puts stdout "$soso pick_dialog = [$soso pick_dialog]"

puts stdout "$w comp.imag comp. = [$w component image component]"

     # get instances of AstroCat
     set test1 [itcl::find objects]
     set a [split $test1 " " ]
     set b [lsearch -glob $a *AstroCat*]
     set a [lindex $a $b]
     set b [split $a "::" ]
     set a [llength $b]
     set b [lindex $b [expr $a -1] ]
     set a [split $b "."]
     set sc ".[lindex $a 1].[lindex $a 2]"
     puts stdout "Instances: >$sc<"
     # finds only the first of many

     set sc [itcl::find objects -class ::skycat::SkyCat]
     # should be e.g. ::.skycat1
     set w [$sc component]
     # should be ::.skycat1.image
     # puts stdout "class objects=$sc, components=$w"
     puts stdout "view components = [$sc component view]"
     puts stdout "image components = [$sc component image]"
     catch {puts stdout "comp view comp [$sc component view component]"}
     catch {puts stdout "comp imag comp [$sc component image component]"}
     # set ol [itcl::find objects]
     # set ol [split $ol " "]
     # set ll [llength $ol]
     # for {set i 0} { $i< $ll} {incr i 1} {
     #     puts stdout [lindex $ol $i]
     #     }

#puts stdout [info context]
#puts stdout "all     -> [info namespace all ]"
#puts stdout "children-> [info namespace children ]"
#puts stdout "parent  -> [info namespace parent ]"

    #----list the functions
    # set f [$sc info function]
    #    set f [split $f " "]
    #    set l [llength $f]
    #    for {set i 0} {$i< $l} { incr i 1} {
    #        # if {[string match [lindex $f $i] *rem* ]==1} {
    #        puts stdout [lindex $f $i]
    #        # }
    #        }
    # puts stdout [set f [$sc info function pick_object] ]

     #---list the variables
     puts stdout ">>>>[itcl::find  objects]"

     set f [::fors::FORS::my info variable]
         set f [split $f " "]
         set l [llength $f]
         for {set i 0} {$i< $l} { incr i 1} {
             # if {[string length [$sc info variable [lindex $f $i] ] ]<100} {
              puts stdout [$sc info variable [lindex $f $i] ]
              #  }
             }


}

    #-----------------------------------------------------
    # FORS W.Hummel 6.10.97 USM
    # pop up a dialog for the FORS instrument 
    # 1. parameter : mos, pmos .. or other instrumental modes
    # 2. parameter : menu widget
    #-----------------------------------------------------
     
  #--------------------
  proc f_on  {w f_number mode m} {
  #--------------------
         # w        =  skycat pointer 
         # f_number =  (we forgot the purpose of this variable, 
         #             its not used in any way)
         # mode     = instrumental mode (e.g. mos, lss)
         # m        = menu button pointer 

         # check the skycat version 

	 global env
#         set s_v [eval skycat_version]
         set s_v 3.0

         if {[string compare $s_v $env(SKYCAT_VER)]} {

        set wid .skycat1.dm         
        catch {destroy $wid}
        set sv [DialogWidget $wid \
           -text "SkyCat Version Error:\n\
                  Found : $s_v\n\
                  Required: $env(SKYCAT_VER)" \
           -bitmap warning \
           -default -1 \
           -buttons {OK Cancel} ]

        if { [$sv activate] } {
             # 'Cancel' was pressed
             return 
             }
           }
        

         # disable the skycat pickobject button 
         set p [$w component view]
         $p.m  entryconfigure 3 -state disabled

         # puts stdout "f_on $w $f_number $mode $m"
         set canvas_ [lindex [$w component image info variable canvas_] 4]
         set image_  [lindex [$w component image info variable image_ ] 4]


         # ask for labels option 
         # since checkbutton enclosed in menu
         # no direct access to deselect possible
                set loc_var_name [$m.layout entrycget 14 -variable]
                global $loc_var_name
                set doit [concat set local $$loc_var_name]
                eval $doit
                # puts stdout $local
                set loc_BPM_name [$m.layout entrycget 13 -variable]
                global $loc_BPM_name
                set doit [concat set l_BPM $$loc_BPM_name]
                eval $doit

         # enable/disable menu buttons 

         $m entryconfigure 0 -state disabled; # disable 'create mask' 
         $m entryconfigure 1 -state normal;   # enable  'reset mask'
         $m entryconfigure 2 -state normal;   # enable  'load mask'
         $m entryconfigure 3 -state normal;   # enable  'delete mask'
         $m.reftarg entryconfigure 6 -state disabled;   # enable MOA10ALI switch 
                         # 4 is the separator

         if {[string match "*mos" $mode]} {
         $m entryconfigure 5 -state normal;   # enable 'AutoPos' for (P)MOS
         $m.reftarg entryconfigure 6 -state normal;          # enable MOA10ALI switch 
            }

         if {[string match "mxu" $mode]} {
         $m entryconfigure 5 -state normal;   # enable 'AutoPos' for MXU
         $m.autopos entryconfigure 1 -state disabled;   # disable shift
         $m.autopos entryconfigure 2 -state disabled;   # disable rotate 
         $m.autopos entryconfigure 3 -state disabled;   # disable all
         $m.layout.sw entryconfigure 19 -state disabled;   # disable slit_width 30''
         $m.layout.sw entryconfigure 20 -state disabled;   # disable slit_width 60''
         $m.layout.sw entryconfigure 21 -state disabled;   # disable slit_width all''
            }

         $m entryconfigure 6 -state normal;   # enable 'Save mos.1.pn.fims'
         $m entryconfigure 7 -state normal;   # enable 'reference targets'  
         $m entryconfigure 8 -state normal;   # enable 'demo' 

                         # 9 is separator

         $m entryconfigure 10 -state normal;  # enable 'Config-Layout' 

         # --- submenu : LAYOUT-CONFIG---
            $m.layout entryconfigure 0 -state normal; # tear-off  


         # enable/disable mask-dependent sub-menu buttons 
         if {[string match "*mos" $mode] || 
             [string match "*mxu*" $mode]} {  
            # MOS/PMOS  Slit Width and Length
            # 0 is tear-off entry 
            $m.layout entryconfigure 1 -state normal; # mos width
            $m.layout entryconfigure 2 -state normal; # mos length
            $m.layout entryconfigure 5 -state normal; # wave 
            $m.layout entryconfigure 7 -state normal; # filter
            $m.layout entryconfigure 8 -state normal; # grism
            $m.layout entryconfigure 15 -state normal; # center-slit 
            }

         if {[string match "*echs*" $mode]} {
            $m.layout entryconfigure 1 -state normal; # mos width
            $m.layout entryconfigure 8 -state normal; # grism
            $m.layout entryconfigure 15 -state normal; # center-slit 
         $m.layout.sw entryconfigure 18 -state disabled;   # disable slit_width 22''
         $m.layout.sw entryconfigure 19 -state disabled;   # disable slit_width 30''
         $m.layout.sw entryconfigure 20 -state disabled;   # disable slit_width 60''
         $m.layout.sw entryconfigure 21 -state disabled;   # disable slit_width all''
            }

         if {[string compare $mode "lss"] == 0} {
            $m.layout entryconfigure 3 -state normal; # lss width
            $m.layout entryconfigure 5 -state normal
            $m.layout entryconfigure 7 -state normal
            $m.layout entryconfigure 8 -state normal
            $m.layout entryconfigure 15 -state normal; # center-slit 
            }
         if {[string compare $mode "hits"] == 0} {
            $m.layout entryconfigure 4 -state normal; # hits width
            $m.layout entryconfigure 5 -state normal
            $m.layout entryconfigure 7 -state normal
            $m.layout entryconfigure 8 -state normal
            $m.layout entryconfigure 15 -state normal; # center-slit 
            }

         $m.layout entryconfigure 10 -state normal;  # rot step
         $m.layout entryconfigure 11 -state normal;  # color 
         $m.layout entryconfigure 12 -state normal;  # tranparency

         $m.layout entryconfigure 13 -state normal; # BadPixelMap  
         $m.layout entryconfigure 14 -state normal; # 'Labels' 
         $m.layout entryconfigure 16 -state normal; # project name 
         $m.layout entryconfigure 17 -state normal; # save directory

             $m.layout.sh invoke 0 



         # enable the collimator button for PMOS
         $m.layout entryconfigure 6 -state normal

	 set which_fors $env(FORS_INST)
	 if {$which_fors == 2} {
	     if {[string compare $mode mos] == 0} {
		 $m.layout entryconfigure 6 -state disabled
	     }
	 }
         if {[string compare $mode pmos] == 0 || 
	     [string compare $mode hits] == 0 } {
             $m.layout entryconfigure 6 -state disabled
         }

         # set the radio buttons back to default

         $m.layout.sw invoke 6; # default MOS slit width 
         $m.layout.sl invoke 0; # default MOS slit length  
         $m.layout.ls invoke 3; # default long slit
         $m.layout.ht invoke 3; # default hits slit
         $m.layout.sh invoke 0; # default collimator 
      $m.layout.fi.m5 invoke 9; # default filter     
         $m.layout.gr invoke 9; # default grism      
         $m.layout.rs invoke 5; # default rot step 
         $m.layout.cs invoke 0; # default color set 
       # $m.layout.st invoke 0; # default transparency 
         

         # create one mask 
         # ........................................................
         fors::FORS::FORSInit $canvas_ $image_ $m $mode 1  $local $l_BPM
         # ........................................................
         # modify label of Save button
         $m entryconfigure 6 -label "Save '\
             [fors::FORS::my_mask cget -modus]'."
      # puts stdout "f_on: end"
      }

      #--------------------
      proc f_off {w f_number mode m} {
      #--------------------
         # w        =  skycat pointer 
         # f_number = 
         # mode     = instrumental mode (e.g. mos, lss)
         # m        = menu button pointer 

   set ln {none}
  catch { set ln [lindex [split [::fors::FORS::my_mask info variable last_save_fn] " "] 4] }
  # puts stdout "last save name = $ln"
  # puts stdout "name length = [string length $ln]"

  if { [string length $ln] < 3} {
     # warning, you want to delete mask but setup not yet saved
        set wid .skycat1.dem
        catch {destroy $wid}
        set dew [DialogWidget $wid \
           -text "No Setup Saved !\n\n\
                  Really Delete Mask ?" \
           -bitmap warning \
           -default -1 \
           -buttons {Yes No} ]
        if { [$dew activate] } return
     }
  


         # enable the skycat pickobject button 
         set p [$w component view]
         $p.m  entryconfigure 3 -state normal

         set canvas_ [lindex [$w component image info variable canvas_] 4]
         set image_  [lindex [$w component image info variable image_ ] 4]

         # delete object FORS::my_mask
         # ..................................................
         fors::FORS::FORSInit $canvas_ $image_ $m $mode 0 0 0
         # ..................................................
         # set the radio buttons back to default
         $m.layout.sw invoke 6; # default MOS slit width 
         $m.layout.sl invoke 0; # default MOS slit length  
         $m.layout.ls invoke 3; # default long slit
         $m.layout.ht invoke 3; # default hits slit
         $m.layout.sh invoke 0; # default collimator 
      $m.layout.fi.m5 invoke 9; # default filter     
         $m.layout.gr invoke 9; # default grism      
         $m.layout.rs invoke 5; # default rot step 
         $m.layout.cs invoke 0; # default color set 
       # $m.layout.st invoke 0; # default transparency 

         $m entryconfigure 0 -state normal;       # create mask
         $m entryconfigure 1 -state disabled;     # reset mask 
         $m entryconfigure 2 -state disabled;     # load mask 
         $m entryconfigure 3 -state disabled;     # delete mask
                        #  4 = separator 
         $m entryconfigure 5 -state disabled;     # autopos 
         $m entryconfigure 6 -state disabled -label "Save MOD1.targ"
         $m entryconfigure 7 -state disabled;     # reference targets
         $m entryconfigure 8 -state disabled;     # demo
                         # 9 is the separator 
         $m entryconfigure 10 -state disabled;     # config
         # 0,1=MOS, 2=LSS 3=HITS
         $m.layout entryconfigure 0  -state disabled;  # tear-off
         $m.layout entryconfigure 1  -state disabled;  # MOS 
         $m.layout entryconfigure 2  -state disabled;  # MOS
         $m.layout entryconfigure 3  -state disabled;  # LSS
         $m.layout entryconfigure 4  -state disabled;  # HITS
         $m.layout entryconfigure 5  -state disabled;  # Wave
         $m.layout entryconfigure 6  -state disabled;  # coll
         $m.layout entryconfigure 7  -state disabled;  # filter 
         $m.layout entryconfigure 8  -state disabled;  # grism  
                                # 9 = separator 
         $m.layout entryconfigure 10 -state disabled;  # rot step
         $m.layout entryconfigure 11 -state disabled;  # color 
         $m.layout entryconfigure 12 -state disabled;  # transparency
         $m.layout entryconfigure 13 -state disabled;  # BadPixelMap 
         $m.layout entryconfigure 14 -state disabled;  # labels 
         $m.layout entryconfigure 15 -state disabled;  # center slit
         $m.layout entryconfigure 16 -state disabled;  # project name 
         $m.layout entryconfigure 17 -state disabled;  # setup dir  

         # for echs
            $m.layout.gr entryconfigure 0 -state normal; # single grisms
            $m.layout.gr entryconfigure 1 -state normal; # single grisms
            $m.layout.gr entryconfigure 2 -state normal; # single grisms
            $m.layout.gr entryconfigure 3 -state normal; # single grisms

         # for mxu 
         $m.layout.sw entryconfigure 18 -state normal;   # enable slit_width 22''
         $m.layout.sw entryconfigure 19 -state normal;   # enable slit_width 30''
         $m.layout.sw entryconfigure 20 -state normal;   # enable slit_width 60''
         $m.layout.sw entryconfigure 21 -state normal;   # enable slit_width all''
         # for non (p)mos
         $m.reftarg   entryconfigure 6  -state disabled;   # enable MOS10ali 
         $m.autopos entryconfigure 1 -state normal;   # enable shift
         $m.autopos entryconfigure 2 -state normal;   # enable rotate 
         $m.autopos entryconfigure 3 -state normal;   # enable all
         }

 #---------------
 proc f_con_sav {m} {
 #---------------
  fors::FORS::my_mask save_TPX 
  set a [lindex [fors::FORS::my_mask info variable exposure_counter] 4]
  set b [fors::FORS::my_mask cget -modus]
  set c [fors::FORS::my_mask cget -slit_width]
  #puts stdout ">>$a<<  >>$b<<"
  if {[string compare $c open]==0} {
     if {[string compare $b mos]==0} {
        $m entryconfigure 6 -label "Save 'ims.$a'"
        } else {
        $m entryconfigure 6 -label "Save 'pims.$a'"
        }
     } else {     
     $m entryconfigure 6 -label "Save '$b.$a'"
     }

  # set b [fors::FORS::my_mask cget -filter]
  # puts stdout "Filter : $b$"
  # $m.layout entryconfigure 5 -label "Filter : $b$"

  # set b [fors::FORS::my_mask cget -grism]
  # puts stdout "Grism : $b$"
  # $m.layout entryconfigure 6 -label "Grism : $b$"
  }

# this is a copy of skycat_about with some modifications
# --------------------
proc fors_about {w_} {
# --------------------
  global fors_logo ::about_fors ::env
  # look for home directory
  set file $env(FIMSROOT)
  append file "/bitmaps/fors-logo.gif"
  # puts stdout $file

  # create the FoRS logo
  if {[catch {
    # create a dummy rtdimage to make sure the colormap is initialized before
    # the logo is created to avoid color problems...
    image delete [image create rtdimage]
    set fors_logo [image create photo \
                         -format GIF \
                         -file $file]
  } msg]} {
  }

# set the text string for "About FIMS ..."
set vf [eval fims_version]
set version [lindex $vf 0]
set last_day [lindex $vf 1]
set last_time [lindex $vf 2]
set about_fors "\
FORS Instrumental Mask Simulator (FIMS) 

cmm $version 

Copyright (C ) 1998 ESO - European Southern Observatory

usg@eso.org

Author:

Wolfgang Hummel 

USM - University Sternwarte Munich

Last modification: $last_day  $last_time
"


  # the rest is original      
  if {! [winfo exists $w_.image.about]} {
      if {! [info exists fors_logo] || "fors_logo" == ""} {
          set bitmap info
          set logo ""
      } else {
          set bitmap ""
          set logo $fors_logo
      }
      catch {destroy $w_.image.about}
      DialogWidget $w_.image.about \
          -image $logo \
          -bitmap $bitmap \
          -messagewidth 6i \
          -justify center \
          -text $about_fors
  }
  $w_.image.about activate
}






# this is a copy of skycat_about with some modifications
# --------------------
proc masks_about  {w_} {
# --------------------
  global fors_logo  ::env 
  # look for home directory
  set file $env(FIMSROOT)
  append file "/bitmaps/fors-logo-small.gif"
  # puts stdout $file

  # create the FoRS logo
  if {[catch {
    # create a dummy rtdimage to make sure the colormap is initialized before
    # the logo is created to avoid color problems...
    image delete [image create rtdimage]
    set fors_logo [image create photo \
                         -format GIF \
                         -file $file]
  } msg]} {
  }
# set the text string for "About Masks..."
set about_masks "
     MASKID\t\t MASK NAME\t\t MODE\t description\n
---------------------------------------------------------------\n"
set contents ""

set myfile $env(FIMSROOT)

append myfile "/src/standard_mask_list"

     if {[file exists $myfile] & [file readable $myfile]} {
        set fileId [open $myfile r ]
        # puts stdout "$myfile open"
        # as long as there are lines in the file
        while {[gets $fileId line_org] >= 0} {
              if {[string match #* $line_org]} {
                 # ignore comment lines
               } else {
               append contents $line_org
               append contents "\n"
               }
               }
            } else {
            puts stdout "could not open $myfile"
            }

append about_masks $contents 

  # the rest is original      
  if {! [winfo exists $w_.image.about]} {
      if {! [info exists fors_logo] || "fors_logo" == ""} {
          set bitmap info
          set logo ""
      } else {
          set bitmap ""
          set logo $fors_logo
      }
      catch {destroy $w_.image.about}
      TextDialog   $w_.image.about \
          -image $logo \
          -bitmap $bitmap \
          -justify center  \
          -textheight 20 \
          -contents $about_masks \
          -text "FORS2-MXU Technical Mask Info"
  }
  $w_.image.about activate
}



# this is a copy of skycat_about with some modifications
# --------------------
proc grism_about {w_} {
# --------------------
  global fors_logo  ::env 
  # look for home directory
  set file $env(FIMSROOT)
  append file "/bitmaps/fors-logo-small.gif"
  # puts stdout $file

  # create the FoRS logo
  if {[catch {
    # create a dummy rtdimage to make sure the colormap is initialized before
    # the logo is created to avoid color problems...
    image delete [image create rtdimage]
    set fors_logo [image create photo \
                         -format GIF \
                         -file $file]
  } msg]} {
  }

# set the text string for "About Grism..."
set about_grism "
     GRISM              FILTER              dis      cen    min    max\n\
-----------------------------------------------------------------------\n"
set contents ""

catch { set contents \
[lindex [fors::FORS::my_mask info variable disp_string] 4] }

set inst "No Instrument selected"
catch { set inst [fors::FORS::my_mask cget -instrument] }

append about_grism $contents 

  # the rest is original      
  if {! [winfo exists $w_.image.about]} {
      if {! [info exists fors_logo] || "fors_logo" == ""} {
          set bitmap info
          set logo ""
      } else {
          set bitmap ""
          set logo $fors_logo
      }
#     DialogWidget $w_.image.about 
      catch {destroy $w_.image.about}
      TextDialog   $w_.image.about \
          -image $logo \
          -bitmap $bitmap \
          -justify center  \
          -text $inst \
          -textheight 20 \
          -contents $about_grism
  }
  $w_.image.about activate
}





#
# ___oOo___

